@@ -1,6 +1,6 @@
 /*
 * this file is part of the oxygen gtk engine
-* Copyright (c) 2010 Hugo Pereira Da Costa <hugo.pereira@free.fr>
+* Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
 *
 * This  library is free  software; you can  redistribute it and/or
 * modify it  under  the terms  of the  GNU Lesser  General  Public
@@ -33,14 +33,13 @@ namespace Oxygen
 
     //_________________________________________________________
     BackgroundHintEngine::BackgroundHintEngine( Animations* animations ):
-        BaseEngine( animations ),
-        _useBackgroundGradient( true )
+        BaseEngine( animations )
     {
 
-        // create background gradient atom
         #ifdef GDK_WINDOWING_X11
+        // create background gradient atom
         GdkDisplay *display( gdk_display_get_default () );
-        if( display )
+        if( display && GDK_IS_X11_DISPLAY( display ) )
         {
 
             _backgroundGradientAtom = XInternAtom( GDK_DISPLAY_XDISPLAY( display ), "_KDE_OXYGEN_BACKGROUND_GRADIENT", False);
@@ -59,9 +58,6 @@ namespace Oxygen
     bool BackgroundHintEngine::registerWidget( GtkWidget* widget, BackgroundHints hints )
     {
 
-        // check enable state
-        if( !enabled() ) return false;
-
         #ifdef GDK_WINDOWING_X11
 
         // get associated top level widget
@@ -80,7 +76,7 @@ namespace Oxygen
 
         // set hint
         GdkDisplay *display( gtk_widget_get_display( topLevel ) );
-        if( _useBackgroundGradient && display && _backgroundGradientAtom && (hints&BackgroundGradient) )
+        if( display && _backgroundGradientAtom && (hints&BackgroundGradient) )
         {
             unsigned long uLongValue( true );
             XChangeProperty(
@@ -140,7 +136,6 @@ namespace Oxygen
 
     }
 
-
     //____________________________________________________________________
     bool BackgroundHintEngine::contains( GtkWidget* widget ) const
     {
@@ -151,6 +146,6 @@ namespace Oxygen
         return false;
         #endif
 
-    }
+     }
 
 }
