@@ -34,17 +34,46 @@
 
 namespace Oxygen
 {
-    GQuark Gtk::Quarks::_rcStyle = 0L;
+
+    //_____________________________________________________________________________
+    std::ostream& operator << ( std::ostream& out, const GtkWidgetPath* path )
+    {
+        if( !path )
+        {
+
+            out << " (null)";
+
+        } else {
+
+            for( gint pos=0; pos<gtk_widget_path_length( path ); ++pos )
+            {
+                const char* name( g_type_name( gtk_widget_path_iter_get_object_type( path, pos ) ) );
+                if(!name) break;
+                out << "/" << name;
+            }
+
+        }
+
+        return out;
+
+    }
+
+
 
     //____________________________________________________________
     void Gtk::gtk_container_adjust_buttons_state(GtkContainer* container,gpointer data)
     {
         if(GTK_IS_BUTTON(container))
         {
-            int x,y;
+
+            int x(0),y(0);
             GtkWidget* button=GTK_WIDGET(container);
             GtkAllocation allocation( gtk_widget_get_allocation( button ) );
-            gtk_widget_get_pointer(button,&x,&y);
+
+            GdkDeviceManager* manager( gdk_display_get_device_manager( gtk_widget_get_display( button ) ) );
+            GdkDevice* pointer( gdk_device_manager_get_client_pointer( manager ) );
+            gdk_window_get_device_position( gtk_widget_get_window( button ), pointer, &x, &y, 0L);
+
             if( !(x>0 && y>0 &&
                 x < allocation.width &&
                 y < allocation.height) && gtk_widget_get_state(button)==GTK_STATE_ACTIVE )
@@ -63,12 +92,16 @@ namespace Oxygen
     }
 
     //____________________________________________________________
-    bool Gtk::gtk_widget_has_custom_background( GtkWidget* widget, GtkStateType state )
+    bool Gtk::gtk_widget_path_has_type( const GtkWidgetPath* path, GType type )
     {
 
-        // loop over all parents, recursively
-        for( GtkWidget* parent = widget; parent; parent = gtk_widget_get_parent( parent ) )
-        { if( gtk_widget_get_modifier_style(parent)->color_flags[state]&GTK_RC_BG ) return true; }
+        if( !path ) return false;
+        for( gint pos=0; pos<gtk_widget_path_length( path ); ++pos )
+        {
+            const GType local( gtk_widget_path_iter_get_object_type( path, pos ) );
+            if( local == type || g_type_is_a( local, type ) )
+            { return true; }
+        }
 
         return false;
 
@@ -183,18 +216,32 @@ namespace Oxygen
         if( !window ) return false;
 
         if( !gdk_default_screen_is_composited() ) return false;
-        return gdk_visual_has_rgba( gdk_drawable_get_visual( GDK_DRAWABLE( window ) ) );
+        return gdk_visual_has_rgba( gdk_window_get_visual( window ) );
 
     }
 
     //________________________________________________________
     bool Gtk::gdk_visual_has_rgba( GdkVisual* visual )
     {
-        return
-            visual->depth == 32 &&
-            visual->red_mask   == 0xff0000 &&
-            visual->green_mask == 0x00ff00 &&
-            visual->blue_mask  == 0x0000ff;
+        // check depth
+        if( gdk_visual_get_depth( visual ) != 32 ) return false;
+
+        // check red pixel
+        guint32 redMask;
+        gdk_visual_get_red_pixel_details( visual, &redMask, 0L, 0L );
+        if( redMask != 0xff0000 ) return false;
+
+        // check green pixel
+        guint32 greenMask;
+        gdk_visual_get_green_pixel_details( visual, &greenMask, 0L, 0L );
+        if( greenMask != 0x00ff00 ) return false;
+
+        // check blue pixel
+        guint32 blueMask;
+        gdk_visual_get_blue_pixel_details( visual, &blueMask, 0L, 0L );
+        if( blueMask != 0x0000ff ) return false;
+
+        return true;
 
     }
 
@@ -279,10 +326,7 @@ namespace Oxygen
 
     //________________________________________________________
     bool Gtk::gtk_button_is_header( GtkWidget* widget )
-    {
-        if( !GTK_IS_BUTTON( widget ) ) return false;
-        return gtk_parent_tree_view( widget ) || gtk_widget_find_parent( widget, "GimpThumbBox" );
-    }
+    { return GTK_IS_BUTTON( widget ) && gtk_parent_tree_view( widget ); }
 
     //________________________________________________________
     bool Gtk::gtk_button_is_in_path_bar( GtkWidget* widget )
@@ -404,15 +448,7 @@ namespace Oxygen
     }
 
     //________________________________________________________
-    bool Gtk::gtk_combobox_is_popup( GtkWidget* widget )
-    {
-        // check types and path
-        if( !GTK_IS_WINDOW(widget) ) return false;
-        return Gtk::gtk_widget_path( widget ) == "gtk-combobox-popup-window";
-    }
-
-    //________________________________________________________
-    bool Gtk::gtk_combo_is_viewport( GtkWidget* widget )
+    bool Gtk::gtk_combobox_is_viewport( GtkWidget* widget )
     {
         if( !GTK_IS_VIEWPORT(widget) ) return false;
         static const std::string match( "gtk-combo-popup-window" );
@@ -420,21 +456,13 @@ namespace Oxygen
     }
 
     //________________________________________________________
-    bool Gtk::gtk_combo_is_frame( GtkWidget* widget )
+    bool Gtk::gtk_combobox_is_frame( GtkWidget* widget )
     {
         if( !GTK_IS_FRAME(widget) ) return false;
         static const std::string match( "gtk-combo-popup-window" );
         return Gtk::gtk_widget_path( widget ).substr( 0, match.size() ) == match;
     }
 
-    //________________________________________________________
-    bool Gtk::gtk_combo_is_popup( GtkWidget* widget )
-    {
-        if( !GTK_IS_WINDOW(widget) ) return false;
-        static const std::string match( "gtk-combo-popup-window" );
-        return Gtk::gtk_widget_path( widget ) == match;
-    }
-
     //________________________________________________________
     bool Gtk::gtk_combobox_appears_as_list( GtkWidget* widget )
     {
@@ -443,14 +471,6 @@ namespace Oxygen
         return (bool) appearsAsList;
     }
 
-    //________________________________________________________
-    bool Gtk::gtk_is_tooltip( GtkWidget* widget )
-    {
-        if( GTK_IS_TOOLTIP(widget) ) return true;
-        const std::string path(Gtk::gtk_widget_path( widget ));
-        return  path == "gtk-tooltip" || path == "gtk-tooltips";
-    }
-
     //________________________________________________________
     bool Gtk::gtk_notebook_tab_contains( GtkWidget* widget, int tab, int x, int y )
     {
@@ -514,11 +534,14 @@ namespace Oxygen
     int Gtk::gtk_notebook_find_first_tab( GtkWidget* widget )
     {
 
-        if( !GTK_IS_NOTEBOOK( widget ) ) return 0;
+        // TODO: reimplement with gtk+3.0
+        return 0;
 
-        // cast to notebook
-        GtkNotebook* notebook( GTK_NOTEBOOK( widget ) );
-        return g_list_position( notebook->children, notebook->first_tab );
+//         if( !GTK_IS_NOTEBOOK( widget ) ) return 0;
+//
+//         // cast to notebook
+//         GtkNotebook* notebook( GTK_NOTEBOOK( widget ) );
+//         return g_list_position( notebook->children, notebook->first_tab );
 
     }
 
@@ -626,12 +649,7 @@ namespace Oxygen
             if( !page ) continue;
 
             GtkWidget* label( gtk_notebook_get_tab_label( notebook, page ) );
-
-            #if GTK_CHECK_VERSION(2, 20, 0)
             if( label && !gtk_widget_get_mapped( label ) ) return true;
-            #else
-            if( label && !GTK_WIDGET_MAPPED( label ) ) return true;
-            #endif
         }
 
         return false;
@@ -723,6 +741,7 @@ namespace Oxygen
         if( h ) *h = -1;
 
         if( !( window && GDK_IS_WINDOW( window ) ) ) return false;
+        if( gdk_window_get_window_type( window ) == GDK_WINDOW_OFFSCREEN ) return false;
 
         // get window size and height
         if( frame ) gdk_toplevel_get_frame_size( window, w, h );
@@ -747,6 +766,7 @@ namespace Oxygen
         // get window
         GdkWindow* window( gtk_widget_get_parent_window( widget ) );
         if( !( window && GDK_IS_WINDOW( window ) ) ) return false;
+        if( gdk_window_get_window_type( window ) == GDK_WINDOW_OFFSCREEN ) return false;
 
         if( frame ) gdk_toplevel_get_frame_size( window, w, h );
         else gdk_toplevel_get_size( window, w, h );
@@ -829,8 +849,16 @@ namespace Oxygen
 
         if( GdkWindow* topLevel = gdk_window_get_toplevel( window ) )
         {
-            gdk_drawable_get_size( topLevel, w, h );
-        } else gdk_drawable_get_size( window, w, h );
+
+            if( w ) *w = gdk_window_get_width( topLevel );
+            if( h ) *h = gdk_window_get_height( topLevel );
+
+        } else {
+
+            if( w ) *w = gdk_window_get_width( window );
+            if( h ) *h = gdk_window_get_height( window );
+
+        }
 
         return;
 
@@ -847,12 +875,24 @@ namespace Oxygen
             return;
         }
 
-        if( GdkWindow* topLevel = gdk_window_get_toplevel( window ) )
+        GdkWindow* topLevel = gdk_window_get_toplevel( window );
+        if( topLevel && GDK_IS_WINDOW( topLevel ) )
         {
-            GdkRectangle rect = {0, 0, -1, -1};
-            gdk_window_get_frame_extents( topLevel, &rect );
-            if( w ) *w = rect.width;
-            if( h ) *h = rect.height;
+            if( gdk_window_get_window_type( topLevel ) == GDK_WINDOW_OFFSCREEN )
+            {
+
+                if( w ) *w = gdk_window_get_width(topLevel);
+                if( h ) *h = gdk_window_get_height(topLevel);
+
+            } else {
+
+                GdkRectangle rect = {0, 0, -1, -1};
+                gdk_window_get_frame_extents( topLevel, &rect );
+
+                if( w ) *w = rect.width;
+                if( h ) *h = rect.height;
+
+            }
         }
 
         return;
@@ -966,25 +1006,12 @@ namespace Oxygen
         if( x ) *x = 0;
         if( y ) *y = 0;
 
-        // get bin window
-        #if GTK_CHECK_VERSION( 2, 20, 0 )
-        GdkWindow* binWindow( gtk_viewport_get_bin_window( viewport ) );
-        #else
-        GdkWindow* binWindow( viewport->bin_window );
-        #endif
-
+        // get windows and derive offsets
         gint xBin(0), yBin(0);
-        gdk_window_get_geometry( binWindow, &xBin, &yBin, 0, 0, 0 );
-
-        // get view window
-        #if GTK_CHECK_VERSION( 2, 22, 0 )
-        GdkWindow* viewWindow( gtk_viewport_get_view_window( viewport ) );
-        #else
-        GdkWindow* viewWindow( viewport->view_window );
-        #endif
+        gdk_window_get_geometry( gtk_viewport_get_bin_window( viewport ), &xBin, &yBin, 0, 0 );
 
         gint xView(0), yView(0);
-        gdk_window_get_geometry( viewWindow, &xView, &yView, 0, 0, 0 );
+        gdk_window_get_geometry( gtk_viewport_get_view_window( viewport ), &xView, &yView, 0, 0 );
 
         // calculate offsets
         if( x ) *x = xView - xBin;
