@@ -23,8 +23,6 @@
 #include "oxygencoloreffect.h"
 #include "oxygencolorutils.h"
 #include "oxygenfontinfo.h"
-#include "oxygengtkicons.h"
-#include "oxygengtkrc.h"
 #include "oxygentimeline.h"
 #include "config.h"
 
@@ -38,6 +36,7 @@
 #include <fstream>
 #include <iostream>
 #include <sstream>
+#include <unistd.h>
 
 namespace Oxygen
 {
@@ -71,6 +70,7 @@ namespace Oxygen
         _menuHighlightMode( MM_DARK ),
         _windowDragEnabled( true ),
         _windowDragMode( WD_FULL ),
+        _useWMMoveResize( true ),
         _startDragDist( 4 ),
         _startDragTime( 500 ),
         _animationsEnabled( true ),
@@ -93,7 +93,8 @@ namespace Oxygen
         _initialized( false ),
         _kdeColorsInitialized( false ),
         _gtkColorsInitialized( false ),
-        _KDESession( false )
+        _KDESession( false ),
+        _provider( gtk_css_provider_new() )
     {}
 
     //_________________________________________________________
@@ -146,7 +147,7 @@ namespace Oxygen
         gtk_settings_set_long_property( settings, "gtk-alternative-button-order", 1, "oxygen-gtk" );
 
         // clear gtkrc
-        _rc.clear();
+        _css.clear();
 
         // kde globals options
         if( flags & KdeGlobals )
@@ -181,17 +182,23 @@ namespace Oxygen
             generateGtkColors();
         }
 
-        // apply extra programatically set metrics metrics
+        // apply extra programatically set metrics
         loadExtraOptions();
 
         // print generated Gtkrc and commit
         #if OXYGEN_DEBUG
-        std::cerr << "Oxygen::QtSettings::initialize - Gtkrc: " << std::endl;
-        std::cerr << _rc << std::endl;
+        std::cerr << "Oxygen::QtSettings::initialize - GtkCss: " << std::endl;
+        std::cerr << _css << std::endl;
         #endif
 
         // pass all resources to gtk and clear
-        _rc.commit();
+        _css.commit( _provider );
+
+        if( GdkScreen* screen = gdk_screen_get_default() )
+        {
+            gtk_style_context_remove_provider_for_screen( screen, GTK_STYLE_PROVIDER( _provider ) );
+            gtk_style_context_add_provider_for_screen( screen, GTK_STYLE_PROVIDER( _provider ), GTK_STYLE_PROVIDER_PRIORITY_THEME + 10 );
+        }
 
         #if OXYGEN_DEBUG
         std::cerr << "Oxygen::QtSettings::initialize - done. " << std::endl;
@@ -201,6 +208,10 @@ namespace Oxygen
 
     }
 
+    //_________________________________________________________
+    bool QtSettings::runCommand( const std::string& command, char*& result ) const
+    { return g_spawn_command_line_sync( command.c_str(), &result, 0L, 0L, 0L ) && result; }
+
     //_________________________________________________________
     bool QtSettings::loadKdeGlobals( void )
     {
@@ -260,11 +271,12 @@ namespace Oxygen
         PathList out;
 
         // load icon install prefix
-        char* path = 0L;
-        if( g_spawn_command_line_sync( "kde4-config --path config", &path, 0L, 0L, 0L ) && path )
+        gchar* path = 0L;
+        if( runCommand( "kde4-config --path config", path ) && path )
         {
 
             out.split( path );
+            g_free( path );
 
         } else {
 
@@ -290,8 +302,11 @@ namespace Oxygen
         // load icon install prefix
         PathList out;
         char* path = 0L;
-        if( g_spawn_command_line_sync( "kde4-config --path icon", &path, 0L, 0L, 0L ) && path )
-        { out.split( path ); }
+        if( runCommand( "kde4-config --path icon", path ) && path )
+        {
+            out.split( path );
+            g_free( path );
+        }
 
         // make sure defaultKdeIconPath is included in the list
         if( std::find( out.begin(), out.end(), _defaultKdeIconPath ) == out.end() )
@@ -311,7 +326,13 @@ namespace Oxygen
         // make sure that corresponding directory does exist
         struct stat st;
         if( stat( _userConfigDir.c_str(), &st ) != 0 )
-        { mkdir( _userConfigDir.c_str(), S_IRWXU|S_IRWXG|S_IRWXO ); }
+        {
+            #if _POSIX_C_SOURCE
+            mkdir( _userConfigDir.c_str(), S_IRWXU|S_IRWXG|S_IRWXO );
+            #else
+            mkdir( _userConfigDir.c_str() );
+            #endif
+        }
 
         // note: in some cases, the target might exist and not be a directory
         // nothing we can do about it. We won't overwrite the file to prevent dataloss
@@ -507,7 +528,7 @@ namespace Oxygen
         GtkSettings* settings( gtk_settings_get_default() );
 
         gtk_settings_set_string_property( settings, "gtk-icon-theme-name", _kdeIconTheme.c_str(), "oxygen-gtk" );
-        gtk_settings_set_string_property( settings, "gtk-fallback-icon-theme-name", _kdeFallbackIconTheme.c_str(), "oxygen-gtk" );
+        gtk_settings_set_string_property( settings, "gtk-fallback-icon-theme", _kdeFallbackIconTheme.c_str(), "oxygen-gtk" );
 
         // load icon sizes from kde
         // const int desktopIconSize( _kdeGlobals.getOption( "[DesktopIcons]", "Size" ).toInt( 48 ) );
@@ -536,7 +557,11 @@ namespace Oxygen
         addIconTheme( iconThemeList, _kdeIconTheme );
         addIconTheme( iconThemeList, _kdeFallbackIconTheme );
 
-        _rc.merge( _icons.generate( iconThemeList ) );
+        // load translation table
+        _icons.loadTranslations( sanitizePath( std::string( GTK_THEME_DIR ) + "/icons4" ) );
+
+        // generate internal icons factory
+        _icons.generate( iconThemeList );
 
     }
 
@@ -628,141 +653,149 @@ namespace Oxygen
         std::cerr << "Oxygen::QtSettings::generateGtkColors" << std::endl;
         #endif
 
+        /*
+        TODO:
+        look at default values from GtkCssProvider for better style definitions
+        (class based, rather than widget based)
+        */
+
         // customize gtk palette
         _palette.setGroup( Palette::Active );
 
-        // default colors
-        _rc.setCurrentSection( Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", _palette.color( Palette::Window ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[PRELIGHT]", _palette.color( Palette::Window ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[ACTIVE]", _palette.color( Palette::Window ) ) );
+        // default
+        _css.setCurrentSection( Gtk::CSS::defaultSection() );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Window ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::WindowText ) ) );
+        addLinkColors( "[Colors:Window]" );
 
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[SELECTED]", _palette.color( Palette::Selected ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[INSENSITIVE]", _palette.color( Palette::Window ) ) );
+        _css.addSection( "*:selected" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Selected ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::WindowText ) ) );
 
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[ACTIVE]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[SELECTED]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::WindowText ) ) );
+        _css.addSection( "*:hover" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Hover ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::WindowText ) ) );
 
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[NORMAL]", _palette.color( Palette::Base ) ) );
+        _css.addSection( ".highlight" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Focus ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::WindowText ) ) );
 
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[ACTIVE]", _palette.color( Palette::Inactive, Palette::Selected ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[PRELIGHT]", _palette.color( Palette::Base ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[SELECTED]", _palette.color( Palette::Selected ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Base ) ) );
+        _css.addSection( "*:insensitive" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Disabled, Palette::Window ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::Disabled, Palette::WindowText ) ) );
 
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[NORMAL]", _palette.color( Palette::Text ) ) );
+        // entries
+        _css.addSection(
+            ".view, "
+            ".entry, "
+            "GtkEntry" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Base ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::Text ) ) );
+        addLinkColors( "[Colors:View]" );
 
-        if( _inactiveChangeSelectionColor ) _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[ACTIVE]", _palette.color( Palette::Text ) ) );
-        else _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[ACTIVE]", _palette.color( Palette::SelectedText ) ) );
+        _css.addSection(
+            ".view:insensitive, "
+            ".entry:insensitive, "
+            "GtkEntry:insensitive, "
+            "GtkTreeView:insensitive, "
+            "GtkTextView:insensitive, "
+            "GtkIconView:insensitive" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Disabled, Palette::Base ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::Disabled, Palette::Text ) ) );
 
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::Text ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[SELECTED]", _palette.color( Palette::SelectedText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Text ) ) );
+        _css.addSection(
+            ".view:selected, "
+            ".entry:selected, "
+            "GtkEntry:selected, "
+            "GtkTreeView:selected, "
+            "GtkTextView:selected" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Selected ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::SelectedText ) ) );
 
-        // buttons
-        _rc.addSection( "oxygen-buttons-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", _palette.color( Palette::Button ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[ACTIVE]", _palette.color( Palette::Button ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[PRELIGHT]", _palette.color( Palette::Button ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Button ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::ButtonText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[ACTIVE]", _palette.color( Palette::ButtonText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::ButtonText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::ButtonText ) ) );
-        _rc.matchClassToSection( "*Button", "oxygen-buttons-internal" );
-        _rc.matchClassToSection( "GtkOptionMenu", "oxygen-buttons-internal" );
-        _rc.matchWidgetClassToSection( "*<GtkButton>.<GtkLabel>", "oxygen-buttons-internal" );
-        _rc.matchWidgetClassToSection( "*<GtkButton>.<GtkAlignment>.<GtkBox>.<GtkLabel>", "oxygen-buttons-internal" );
+        // sidebars
+        _css.addSection( "GtkAssistant .sidebar" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Disabled, Palette::Base ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::Disabled, Palette::Text ) ) );
 
-        // combobox
-        _rc.addSection( "oxygen-combobox-internal", "oxygen-buttons-internal" );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[NORMAL]", _palette.color( Palette::ButtonText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[ACTIVE]", _palette.color( Palette::ButtonText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::ButtonText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::ButtonText ) ) );
-        _rc.matchWidgetClassToSection( "*<GtkComboBox>.*<GtkCellView>", "oxygen-combobox-internal" );
+         // buttons and comboboxes
+        _css.addSection( "GtkButton" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Button ) ) );
+        addLinkColors( "[Colors:Button]" );
+
+        _css.addSection( "GtkButton:insensitive" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Disabled, Palette::Button ) ) );
+
+        _css.addSection(
+            "GtkButton>GtkLabel, "
+            "GtkButton>GtkAlignment>GtkBox>GtkLabel, "
+            "GtkComboBox>GtkCellView" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::ButtonText ) ) );
+
+        _css.addSection(
+            "GtkButton>GtkLabel:insensitive, "
+            "GtkButton>GtkAlignment>GtkBox>GtkLabel:insensitive, "
+            "GtkComboBox>GtkCellView:insensitive" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::Disabled, Palette::ButtonText ) ) );
 
         // checkboxes and radio buttons
-        _rc.addSection( "oxygen-checkbox-buttons-internal", "oxygen-buttons-internal" );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[ACTIVE]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::WindowText ) ) );
-        _rc.matchWidgetClassToSection( "*<GtkCheckButton>.<GtkLabel>", "oxygen-checkbox-buttons-internal" );
+        _css.addSection( "GtkCheckButton>GtkLabel" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::WindowText ) ) );
 
-        // progressbar labels
-        _rc.addSection( "oxygen-progressbar-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[NORMAL]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[ACTIVE]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::WindowText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::WindowText ) ) );
-
-        _rc.matchClassToSection( "GtkProgressBar", "oxygen-progressbar-internal" );
-
-        // menu items
-        _rc.addSection( "oxygen-menubar-item-internal", "oxygen-menu-font-internal" );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[NORMAL]", _palette.color( Palette::Text ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::WindowText ) ) );
-
-        if( _menuHighlightMode == MM_STRONG )
-        {
-
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::Text ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[SELECTED]", _palette.color( Palette::SelectedText ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[ACTIVE]", _palette.color( Palette::SelectedText ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::WindowText ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[SELECTED]", _palette.color( Palette::SelectedText ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[ACTIVE]", _palette.color( Palette::SelectedText ) ) );
-
-        } else {
-
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::Text ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[SELECTED]", _palette.color( Palette::Text ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[ACTIVE]", _palette.color( Palette::Text ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::WindowText ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[SELECTED]", _palette.color( Palette::WindowText ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[ACTIVE]", _palette.color( Palette::WindowText ) ) );
-
-        }
-        _rc.matchWidgetClassToSection( "*<GtkMenuItem>.<GtkLabel>", "oxygen-menubar-item-internal" );
-
-
-        if( _menuHighlightMode == MM_STRONG )
-        {
-            _rc.addSection( "oxygen-menu-item-internal", "oxygen-menubar-item-internal" );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::SelectedText ) ) );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::SelectedText ) ) );
-            _rc.matchWidgetClassToSection( "*<GtkMenu>.<GtkMenuItem>.<GtkLabel>", "oxygen-menu-item-internal" );
-        }
-
-        // text entries
-        _rc.addSection( "oxygen-entry-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", _palette.color( Palette::Base ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Base ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Base ) ) );
-        _rc.matchClassToSection( "GtkSpinButton", "oxygen-entry-internal" );
-        _rc.matchClassToSection( "GtkEntry", "oxygen-entry-internal" );
-        _rc.matchClassToSection( "GtkTextView", "oxygen-entry-internal" );
-        _rc.matchWidgetClassToSection( "*<GtkComboBoxEntry>.<GtkButton>", "oxygen-entry-internal" );
-        _rc.matchWidgetClassToSection( "*<GtkCombo>.<GtkButton>", "oxygen-entry-internal" );
+        _css.addSection( "GtkCheckButton>GtkLabel:insensitive" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::Disabled, Palette::WindowText ) ) );
 
         // tooltips
-        _rc.addSection( "oxygen-tooltips-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", _palette.color( Palette::Tooltip ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::TooltipText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<int>( "  xthickness", 3 ) );
-        _rc.addToCurrentSection( Gtk::RCOption<int>( "  ythickness", 3 ) );
-        _rc.matchWidgetToSection( "gtk-tooltip*", "oxygen-tooltips-internal" );
+        _css.addSection( "GtkWindow#gtk-tooltip" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Tooltip ) ) );
+        _css.addToCurrentSection( "  padding: 3px;" );
 
-        // special case for google chrome
-        /* based on http://code.google.com/p/chromium/wiki/LinuxGtkThemeIntegration */
-        _rc.addSection( "oxygen-chrome-gtk-frame-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( " ChromeGtkFrame::frame-color", _palette.color( Palette::Window ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( " ChromeGtkFrame::inactive-frame-color", _palette.color( Palette::Window ) ) );
-        _rc.matchClassToSection( "ChromeGtkFrame", "oxygen-chrome-gtk-frame-internal" );
+        _css.addSection( "GtkWindow#gtk-tooltip GtkLabel" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::TooltipText ) ) );
+        addLinkColors( "[Colors:Tooltip]" );
+
+        // rubber band selection (at least for nautilus)
+        // FIXME: is Palette::Selected the proper color? Is 0.35 the proper alpha?
+        _css.addSection( ".view.rubberband,.rubberband" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BORDER_COLOR, _palette.color( Palette::Selected ) ) );
+        {
+            std::ostringstream color;
+            color << "  " << GTK_STYLE_PROPERTY_BACKGROUND_COLOR << ": alpha ("<< (std::string)_palette.color( Palette::Selected ) << ", 0.35);";
+            _css.addToCurrentSection( color.str() );
+        }
+
+        /*
+        this is stupid. Setting the color to all widgets, like above, erases the .info, warning, etc. colors.
+        Need to reset them here. This is copied from gtkcssprovider.c
+        */
+        _css.addSection( ".info" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, "@info_bg_color" ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, "@info_fg_color" ) );
+
+        _css.addSection( ".warning" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, "@warning_bg_color" ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, "@warning_fg_color" ) );
+
+        _css.addSection( ".question" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, "@question_bg_color" ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, "@question_fg_color" ) );
+
+        _css.addSection( ".error" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, "@error_bg_color" ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, "@error_fg_color" ) );
+
+    }
+
+    //_________________________________________________________
+    void QtSettings::addLinkColors( const std::string& section )
+    {
+
+        // link colors
+        const ColorUtils::Rgba linkColor( ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( section, "ForegroundLink" ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( "-GtkWidget-link-color", linkColor ) );
+
+        // visited link colors
+        const ColorUtils::Rgba visitedLinkColor( ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( section, "ForegroundVisited" ) ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( "-GtkWidget-visited-link-color", visitedLinkColor ) );
 
     }
 
@@ -817,34 +850,37 @@ namespace Oxygen
 
         }
 
-        // pass fonts to RC
+        // pass fonts to GtkSettings and css
         if( fonts[FontInfo::Default].isValid() )
         {
             // pass to settings
             GtkSettings* settings( gtk_settings_get_default() );
             gtk_settings_set_string_property( settings, "gtk-font-name", fonts[FontInfo::Default].toString( false ).c_str(), "oxygen-gtk" );
 
-            _rc.setCurrentSection( Gtk::RC::defaultSection() );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::Default].toString() ) );
+            _css.setCurrentSection( Gtk::CSS::defaultSection() );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( "font", fonts[FontInfo::Default].toString( false ) ) );
+
         }
 
         if( fonts[FontInfo::Menu].isValid() )
         {
-            _rc.addSection( "oxygen-menu-font-internal", Gtk::RC::defaultSection() );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::Menu].toString() ) );
-            _rc.matchWidgetClassToSection( "*<GtkMenuItem>.<GtkLabel>", "oxygen-menu-font-internal" );
+            _css.addSection( "GtkMenuItem" );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( "font", fonts[FontInfo::Menu].toString( false ) ) );
         }
 
         if( fonts[FontInfo::ToolBar].isValid() )
         {
-            _rc.addSection( "oxygen-toolbar-font-internal", Gtk::RC::defaultSection() );
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::ToolBar].toString() ) );
-            _rc.matchWidgetClassToSection( "*<GtkToolbar>.*", "oxygen-toolbar-font-internal" );
+            _css.addSection( "GtkToolbar" );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( "font", fonts[FontInfo::ToolBar].toString( false ) ) );
         }
 
         // don't check for section and tag presence - use default font if not present
         _WMFont=FontInfo::fromKdeOption( _kdeGlobals.getValue( "[WM]", "activeFont", "Sans Serif,8,-1,5,50,0,0,0,0,0" ) );
 
+
+        _css.addSection( "GtkAssistant .sidebar .highlight" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( "font-weight", "bold" ) );
+
     }
 
     //_________________________________________________________
@@ -865,6 +901,9 @@ namespace Oxygen
         GtkSettings* settings( gtk_settings_get_default() );
         gtk_settings_set_long_property( settings, "gtk-toolbar-style", toolbarStyle, "oxygen-gtk" );
 
+        // dialog button ordering
+        gtk_settings_set_long_property( settings, "gtk-alternative-button-order", 1, "oxygen-gtk" );
+
         // icons on buttons
         if( _kdeGlobals.getValue( "[KDE]", "ShowIconsOnPushButtons", "true" ) == "false" )
         { gtk_settings_set_long_property( settings, "gtk-button-images", 0, "oxygen-gtk" ); }
@@ -933,6 +972,9 @@ namespace Oxygen
         if( windowDragMode == "WD_MINIMAL" ) _windowDragMode = WD_MINIMAL;
         else _windowDragMode = WD_FULL;
 
+        // use window manager to handle window drag
+        _useWMMoveResize = _oxygen.getOption( "[Style]", "UseWMMoveResize" ).toVariant<std::string>("true") == "true";
+
         // animations
         _animationsEnabled = ( _oxygen.getOption( "[Style]", "AnimationsEnabled" ).toVariant<std::string>("true") == "true" );
         _genericAnimationsEnabled = ( _oxygen.getOption( "[Style]", "GenericAnimationsEnabled" ).toVariant<std::string>("true") == "true" );
@@ -1011,17 +1053,17 @@ namespace Oxygen
 
         // copy relevant options to to gtk
         // scrollbar width
-        _rc.setCurrentSection( Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<int>(
-            "  GtkScrollbar::slider-width",
-            _oxygen.getOption( "[Style]", "ScrollBarWidth" ).toVariant<int>(15) + 1 ) );
+        _css.addSection( "GtkScrollbar" );
+        _css.addToCurrentSection( Gtk::CSSOption<int>(
+            "-GtkRange-slider-width",
+            _oxygen.getOption( "[Style]", "ScrollBarWidth" ).toVariant<int>(15) ) );
 
-        _rc.addToCurrentSection( Gtk::RCOption<bool>("  GtkScrollbar::has-backward-stepper", _scrollBarSubLineButtons > 0 ) );
-        _rc.addToCurrentSection( Gtk::RCOption<bool>("  GtkScrollbar::has-forward-stepper", _scrollBarAddLineButtons > 0 ) );
+        _css.addToCurrentSection( Gtk::CSSOption<bool>("-GtkScrollbar-has-backward-stepper", _scrollBarSubLineButtons > 0 ) );
+        _css.addToCurrentSection( Gtk::CSSOption<bool>("-GtkScrollbar-has-forward-stepper", _scrollBarAddLineButtons > 0 ) );
 
         // note the inversion for add and sub, due to the fact that kde options refer to the button location, and not its direction
-        _rc.addToCurrentSection( Gtk::RCOption<bool>("  GtkScrollbar::has-secondary-backward-stepper", _scrollBarAddLineButtons > 1 ) );
-        _rc.addToCurrentSection( Gtk::RCOption<bool>("  GtkScrollbar::has-secondary-forward-stepper", _scrollBarSubLineButtons > 1 ) );
+        _css.addToCurrentSection( Gtk::CSSOption<bool>("-GtkScrollbar-has-secondary-backward-stepper", _scrollBarAddLineButtons > 1 ) );
+        _css.addToCurrentSection( Gtk::CSSOption<bool>("-GtkScrollbar-has-secondary-forward-stepper", _scrollBarSubLineButtons > 1 ) );
 
         // mnemonics
         GtkSettings* settings( gtk_settings_get_default() );
@@ -1072,35 +1114,94 @@ namespace Oxygen
     void QtSettings::loadExtraOptions( void )
     {
 
-        // deal with pathbar button margins
-        // this needs to be done programatically in order to properly account for RTL locales
-        _rc.addSection( "oxygen-pathbutton-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( "  GtkButton::inner-border = { 2, 2, 1, 0 }" );
+        #if GTK_CHECK_VERSION( 3, 3, 0 )
 
-        if( gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL )
-        {
+        // button padding
+        _css.setCurrentSection( "GtkButton" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "3px 4px 2px" ) );
 
-            _rc.addToCurrentSection( "  GtkToggleButton::inner-border={ 10, 0, 1, 0 }" );
+        // pathbar button margins
+        _css.addSection( "GtkPathBar>GtkToggleButton" );
+        _css.addToCurrentSection(
+            ( gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL ) ?
+            Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "3px 2px 2px 12px;" ):
+            Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "3px 12px 2px 2px;" ) );
 
-        } else {
+        _css.addSection( "NautilusPathBar>GtkToggleButton" );
+        _css.addToCurrentSection(
+            ( gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL ) ?
+            Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "2px 2px 2px 12px;" ):
+            Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "2px 12px 2px 2px;" ) );
 
-            _rc.addToCurrentSection( "  GtkToggleButton::inner-border={ 0, 10, 1, 0 }" );
+        // spinbox buttons
+        _css.addSection( "GtkSpinButton.button" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, "alpha( #900185, 0 )" ) );
+        _css.addToCurrentSection( ( gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL ) ?
+            Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "0px 12px 0px 0px" ):
+            Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "0px 0px 0px 12px" ) );
 
-        }
+        // entries
+        _css.addSection( "GtkEntry" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "4px 7px" ) );
 
-        _rc.matchWidgetClassToSection( "*PathBar.GtkToggleButton", "oxygen-pathbutton-internal" );
+        #else
 
-        // entry margins
-        _rc.addSection( "oxygen-entry-margins-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<int>( "  xthickness", 5 ) );
-        _rc.addToCurrentSection( Gtk::RCOption<int>( "  ythickness", applicationName().isXul() ? 2:1 ) );
-        _rc.matchClassToSection( "GtkEntry", "oxygen-entry-margins-internal" );
+        // pathbar button margins
+        _css.addSection( "GtkPathBar>GtkToggleButton" );
+        _css.addToCurrentSection( (gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL ) ?
+            "  -GtkButton-inner-border: 1px 0px 0px 10px;":
+            "  -GtkButton-inner-border: 1px 10px 0px 0px;" );
 
-        // combobox buttons
-        _rc.addSection( "oxygen-combobox-button-internal", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<int>( "  xthickness", 2 ) );
-        _rc.addToCurrentSection( Gtk::RCOption<int>( "  ythickness", applicationName().isXul() ? 2:0 ) );
-        _rc.matchWidgetClassToSection( "*<GtkComboBox>.<GtkButton>", "oxygen-combobox-button-internal" );
+        // pathbar button margins
+        _css.addSection( "NautilusPathBar>GtkToggleButton" );
+        _css.addToCurrentSection( (gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL ) ?
+            "  -GtkButton-inner-border: 0px 0px 0px 10px;":
+            "  -GtkButton-inner-border: 0px 10px 0px 0px;" );
+
+        // button padding
+        _css.setCurrentSection( Gtk::CSS::defaultSection() );
+        _css.addToCurrentSection( "  -GtkButton-inner-border: 1px 2px 0px;" );
+        _css.addToCurrentSection( "  -GtkCalendar-inner-border: 0px;" );
+
+        // toggle button
+        _css.addSection( "GtkToggleButton" );
+        _css.addToCurrentSection( "  -GtkButton-inner-border: 1px 0px 0px;" );
+
+        // entries
+        _css.addSection( "GtkEntry" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "2px 5px" ) );
+
+        #endif
+
+        #if GTK_CHECK_VERSION( 3, 7, 0 )
+
+        // menu
+        // since gtk 3.7.0, GtkMenu-horizontal-padding and vertical-padding (from gtk.css) are ignored
+        // need to use css padding and border-width instead
+        _css.addSection( "GtkMenu" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "5px 3px" ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BORDER_WIDTH, "0px" ) );
+
+        _css.addSection( "GtkMenu>GtkMenuItem" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "5px 4px" ) );
+
+        _css.addSection( "GtkMenu>GtkSeparatorMenuItem" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "1px 4px" ) );
+
+        #else
+
+        // menu padding
+        _css.setCurrentSection( Gtk::CSS::defaultSection() );
+        _css.addToCurrentSection( "  -GtkMenu-horizontal-padding: 3px;" );
+        _css.addToCurrentSection( "  -GtkMenu-vertical-padding: 5px;" );
+
+        _css.addSection( "GtkMenu>GtkMenuItem" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "5px 1px" ) );
+
+        _css.addSection( "GtkMenu>GtkSeparatorMenuItem" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "1px 1px" ) );
+
+        #endif
 
     }
 
