@@ -72,12 +72,13 @@ namespace Oxygen
         #endif
 
         _styleSetHook.disconnect();
+        _styleUpdatedHook.disconnect();
         _buttonReleaseHook.disconnect();
         _map.disconnectAll();
         _map.clear();
 
         // cursor
-        if( _cursor ) gdk_cursor_unref( _cursor );
+        if( _cursor ) g_object_unref( _cursor );
 
     }
 
@@ -89,8 +90,10 @@ namespace Oxygen
         if( _dragMode!=Disabled )
         {
             _styleSetHook.connect( "style-set", (GSignalEmissionHook)styleSetHook, this );
+            _styleUpdatedHook.connect( "style-updated", (GSignalEmissionHook)styleSetHook, this );
             _buttonReleaseHook.connect( "button-release-event", (GSignalEmissionHook)buttonReleaseHook, this );
         }
+
         _hooksInitialized = true;
 
     }
@@ -326,7 +329,9 @@ namespace Oxygen
             GTK_IS_VIEWPORT( widget ) ||
             GTK_IS_TOOLBAR( widget ) ||
             GTK_IS_MENU_BAR( widget ) ||
-            GTK_IS_NOTEBOOK( widget ) )
+            GTK_IS_NOTEBOOK( widget ) ||
+            GTK_IS_PANED( widget )
+            )
         {
 
             // top-level windows
@@ -578,6 +583,19 @@ namespace Oxygen
             else if( !Style::instance().animations().tabWidgetEngine().contains( widget ) ) return false;
             else return !Style::instance().animations().tabWidgetEngine().isInTab( widget, xLocal, yLocal );
 
+        } else if( GTK_IS_PANED( widget ) ) {
+
+            // compare to event position
+            if( !Gtk::gdk_rectangle_contains( &allocation, xLocal, yLocal ) ) return false;
+
+            // get handle window allocation
+            GtkAllocation handleRect;
+            GdkWindow* handle( gtk_paned_get_handle_window( GTK_PANED( widget ) ) );
+            gdk_window_get_geometry( handle, &handleRect.x, &handleRect.y, &handleRect.width, &handleRect.height );
+
+            // check position against handle rect
+            return !Gtk::gdk_rectangle_contains( &handleRect, xLocal, yLocal );
+
         } else {
 
             // compare to event position
@@ -730,7 +748,6 @@ namespace Oxygen
         _blackList.push_back( "MetaFrames" );
         _blackList.push_back( "SPHRuler" );
         _blackList.push_back( "SPVRuler" );
-        _blackList.push_back( "GtkPlug" );
     }
 
     //________________________________________________________________________________
